@@ -33,7 +33,10 @@ class ConstantDict(object):
 
 class MsgFields(ConstantDict):
     ASNIP = "asnip"
+    ASN = "asn"
+    AS_NAME = "asname"
     ARTIFACT = "artifact"
+    CIDR = "cidr"
     COUNTRY_CODE = "cc"
     LAT = "lat"
     LONG = "long"
@@ -68,10 +71,11 @@ class MsgFields(ConstantDict):
     META_ERR = "e"
     META_MSG = "m"
 
+    TASKS_ASNIP = "asnip_task"
+    TASKS_GEOIP = "geoip_task"
     TASKS_HTTP_STATUS = "http_status_task"
     TASKS_RESOLVER = "resolver_task"
     TASKS_WHOIS = "whois_task"
-    TASKS_GEOIP = "geoip_task"
 
 
 class MsgValues(ConstantDict):
@@ -85,6 +89,7 @@ class MsgErrs(ConstantDict):
     HOSTNAME_UNRESOLVABLE = "HOSTNAME_UNRESOLVABLE::{0}"
     HOSTNAME_INVALID = "HOSTNAME_INVALID::{0}"
 
+    RESULT_FAIL = "TASK_RESULT_FAIL::{0}"
     RESULT_INVALID = "RESULT_INVALID::{0}"
 
     UNSPECIFIED_ERROR = "UNSPECIFIED_ERROR::{0}"
