@@ -7,6 +7,8 @@ import json
 from ..taskresultset import TaskResultSet
 from ..taskresult import TaskResult
 from ..schemas import build_schema
+from ..exceptions import TaskResultFailException
+from ..fields import MsgErrs as MERR
 
 
 log = logging.getLogger(name=__name__)
@@ -105,7 +107,7 @@ class TestTaskResultSet(unittest.TestCase):
         else:
             assert(False)
 
-    def test_missing_val(self):
+    def test_missing_tmoid(self):
         m = TaskResultSet(4, 5000)
         tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
         tr.add("a", "result")
@@ -127,14 +129,14 @@ class TestTaskResultSet(unittest.TestCase):
 
         UnicodeSchema = build_schema({
             "properties": {
-                "z": {"type": "string"}
+                "z": {"type": "string"},
+                u"俺": {"type": "string"},
             }
         })
 
         m = TaskResultSet(4, 5000)
         tr = TaskResult(output, 100, task, "1.1.1",
                         schema=UnicodeSchema)
-        tr.add("a", "result")
         tr.add(key, val)
         tr.finish()
         m.add(tr)
@@ -143,6 +145,109 @@ class TestTaskResultSet(unittest.TestCase):
 
         self.assertEqual(tr_extract.get_data(key), val)
 
+    # test get_output_tmoid_or_none
+
+    def test_return_none_simple(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish()
+        m.add(tr)
+
+        ret_tr, err = m.get_output_tmoid_or_none(self.output_name_1, 100)
+        print ret_tr, err
+        self.assertEqual(ret_tr.get_data("a"), "result")
+        self.assertEqual(err, None)
+
+    def test_return_none_on_tr_error(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish(err="An error")
+        m.add(tr)
+
+        ret_tr, err = m.get_output_tmoid_or_none(self.output_name_1, 100)
+        self.assertIsNone(ret_tr)
+        self.assertIsNotNone(err)
+
+    def test_return_none_on_no_output(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish(err="An error")
+        m.add(tr)
+
+        ret_tr, err = m.get_output_tmoid_or_none("not there", 100)
+        self.assertIsNone(ret_tr)
+        self.assertIsNotNone(err)
+
+    def test_return_none_on_no_tmoid(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish(err="An error")
+        m.add(tr)
+
+        ret_tr, err = m.get_output_tmoid_or_none("not there", 999)
+        self.assertIsNone(ret_tr)
+        self.assertIsNotNone(err)
+
+    # test get_output_tmoid_raise_fail
+
+    def test_raise_fail_simple(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish()
+        m.add(tr)
+
+        ret_tr = m.get_output_tmoid_raise_fail(self.output_name_1, 100)
+        self.assertEqual(ret_tr.get_data("a"), "result")
+
+    def test_raise_fail_on_tr_error(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish(err="An error")
+        m.add(tr)
+
+        try:
+            m.get_output_tmoid_raise_fail(self.output_name_1, 100)
+        except TaskResultFailException as e:
+            self.assertTrue(str(e).startswith(MERR.RESULT_FAIL.format("")))
+        else:
+            assert(False)
+
+    def test_raise_on_no_output(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish(err="An error")
+        m.add(tr)
+
+        try:
+            m.get_output_tmoid_raise_fail("not there", 100)
+        except ValueError:
+            assert(True)
+        else:
+            assert(False)
+
+    def test_raise_on_no_tmoid(self):
+        m = TaskResultSet(4, 5000)
+        tr = TaskResult(self.output_name_1, 100, self.task_name_1, "1.1.1")
+        tr.add("a", "result")
+        tr.finish(err="An error")
+        m.add(tr)
+
+        try:
+            m.get_output_tmoid_raise_fail("not there", 999)
+        except ValueError:
+            assert(True)
+        else:
+            assert(False)
+
+    # loops over outputs
+
     def test_task_item_loop(self):
         TaskLoop1Schema = build_schema({
             "properties": {
@@ -159,12 +264,14 @@ class TestTaskResultSet(unittest.TestCase):
         })
 
         m = TaskResultSet(4, 5000)
-        tr = TaskResult("output1", 100, "task1", "1.1.1", schema=TaskLoop1Schema)
+        tr = TaskResult(
+            "output1", 100, "task1", "1.1.1", schema=TaskLoop1Schema)
         tr.add("y", "a")
         tr.add("z", "b")
         tr.finish()
 
-        tr2 = TaskResult("output2", 100, "task2", "1.1.1", schema=TaskLoop2Schema)
+        tr2 = TaskResult(
+            "output2", 100, "task2", "1.1.1", schema=TaskLoop2Schema)
         tr2.add("y", "hello")
         tr2.add("z", None)
         tr2.finish()
@@ -197,17 +304,20 @@ class TestTaskResultSet(unittest.TestCase):
         })
 
         m = TaskResultSet(4, 5000)
-        tr = TaskResult("output1", 100, "task1", "1.1.1", schema=TaskLoop1Schema)
+        tr = TaskResult(
+            "output1", 100, "task1", "1.1.1", schema=TaskLoop1Schema)
         tr.add("y", {"a": "1a"})
         tr.add("z", "b")
         tr.finish()
 
-        tr2 = TaskResult("output1", 101, "task1", "1.1.1", schema=TaskLoop1Schema)
+        tr2 = TaskResult(
+            "output1", 101, "task1", "1.1.1", schema=TaskLoop1Schema)
         tr2.add("y", {"a": "9z"})
         tr2.add("z", "x")
         tr2.finish()
 
-        tr3 = TaskResult("output2", 100, "task2", "1.1.1", schema=TaskLoop2Schema)
+        tr3 = TaskResult(
+            "output2", 100, "task2", "1.1.1", schema=TaskLoop2Schema)
         tr3.add("a", "hello")
         tr3.add("b", "foo")
         tr3.finish()
@@ -219,8 +329,10 @@ class TestTaskResultSet(unittest.TestCase):
         self.assertEqual(set(m.output_tmoids("output1")), set([100, 101]))
         log.debug(m)
         tr_extract = m.get_output_tmoid("output1", 100, schema=TaskLoop1Schema)
-        tr2_extract = m.get_output_tmoid("output1", 101, schema=TaskLoop1Schema)
-        tr3_extract = m.get_output_tmoid("output2", 100, schema=TaskLoop2Schema)
+        tr2_extract = m.get_output_tmoid(
+            "output1", 101, schema=TaskLoop1Schema)
+        tr3_extract = m.get_output_tmoid(
+            "output2", 100, schema=TaskLoop2Schema)
 
         self.assertEqual(tr_extract.get_data_strict("y"), {"a": "1a"})
         self.assertEqual(tr_extract.get_data_strict("z"), "b")
@@ -303,7 +415,8 @@ class TestTaskResultSet(unittest.TestCase):
         })
 
         trs = TaskResultSet(4, 5000)
-        ipset = TaskResult("output1", 100, "task1", "1.1.1", schema=IpSetSchema)
+        ipset = TaskResult(
+            "output1", 100, "task1", "1.1.1", schema=IpSetSchema)
 
         ip = TaskResult("output1", 100, "task1", "1.1.1", schema=IpSchema)
         ip.add("ip", "1.1.1.1")
